unit TDirect3D;

// Code by Massimo Bardetti - bigben@dircon.co.uk  (12 March 1997)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DDraw, D3D, D3DCaps, D3DTypes, DirectDraw, Direct3d, TExecute, TMatrix, TTexture;

type
  TDirect3DIM = class(TPanel)
  private
    FEBRoot : TExecuteBuffer;
    FMaterials : TList;
    FMatrices : TList;
    FLights : TList;
    FTextures : TList;

    { Private declarations }

    Screenrect : D3DRect;

    binitialized : boolean;
    bFullScreen : boolean;

  protected
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
    { Protected declarations }
  public
    DirectDraw : TDDScreen;
    Direct3d : TD3D;

    iScreenWidth, iScreenHeight : integer;
    iBpp : Integer;
    iApplicationHandle : HWND;

    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor destroy; override;

    function AddExecuteBuffer(EBRoot : TExecuteBuffer; iSize : integer) : TExecuteBuffer;
    function GetEBRoot : TExecuteBuffer;
    function GetEBTagged(EBRoot : TExecuteBuffer; itag : Integer) : TExecuteBuffer;
    procedure RemoveEB(EBdel : TExecuteBuffer);

    procedure Initialize(cm : D3DCOLORMODEL; pbFullScreen : Boolean);
    procedure Finish;
    function Material(n : integer) : T3dMaterial;
    function Matrix(n : integer) : T3dMatrix;
    function Light(n : integer) : T3dLight;
    function Texture(n : integer) : T3dTexture;
    function Execute : boolean;
  published
    { Published declarations }
  end;

procedure Register;

implementation

var
   MatTemp : T3dMaterial;

constructor TDirect3dIM.Create(AOwner : TComponent);
begin
     inherited Create(AOwner);
     FEBRoot := nil;
     DirectDraw := nil;
     bInitialized := false;
end;

destructor TDirect3DIM.Destroy;
var
   i : integer;
begin
     inherited;

     if bInitialized then
        finish;
end;


procedure TDirect3DIM.Initialize(cm : D3DCOLORMODEL; pbFullScreen : Boolean);
var
   vd : D3DViewport;
begin
     FMaterials := TList.Create;
     FLights := TList.Create;
     FTextures := TList.Create;
     FMatrices := TList.Create;

     if not pbFullScreen then
       begin
        DirectDraw := TDDScreen.Create(Width,Height,0,Handle,False);
        iScreenWidth := width;
        iScreenHeight := Height;
       end
     else
        begin
         if iApplicationHandle <> 0 then
            DirectDraw := TDDScreen.Create(iScreenWidth,iScreenHeight,iBpp,iApplicationHandle,True);
         end;


     bFullScreen := pbFullScreen;
     Direct3D := TD3D.Create(DirectDraw);
     Direct3D.SetMode(cm);

     Direct3D.CreateViewPort;

     ZeroMemory(@vd,sizeof(vd));
     vd.dwSize := sizeof(vd);
     vd.dwX := 0;
     vd.dwY := 0;
     vd.dwWidth := iScreenWidth;
     vd.dwHeight := iScreenheight;

     vd.dvScaleX := iScreenWidth / 2.0;
     vd.dvScaleY := iScreenHeight / 2.0;

     vd.dvMaxX := 2.0;
     vd.dvMaxY := 5.0;

     vd.dvMinZ := -5.0;
     vd.dvMaxZ := 100.0;

     DDResult := Direct3D.D3DViewPort.SetViewport(vd);
     if (DDResult <> DD_Ok) then
       begin
        Direct3d.Free;
        DirectDraw.Free;
        raise Exception.Create ( format ('TDirect3D : Cannot Set viewport!! %x', [DDResult]) ) ;
       end;

     Screenrect.x1 := 0;
     Screenrect.y1 := 0;
     Screenrect.x2 := iScreenwidth;
     Screenrect.y2 := iScreenheight;


     bInitialized := True;

end;

procedure TDirect3DIM.Finish;
var
   i : integer;
begin
     i:=0;
     while i < FLights.count do
           begin
                Light(i).Free();
                I := I+ 1;
           end;
     i:=0;
     while i < FMaterials.count do
           begin
                Material(i).Free();
                I := I+ 1;
           end;
     i:=0;
     while i < FTextures.count do
           begin
                Texture(i).Free();
                I := I+ 1;
           end;

     i:=0;
     while i < FMatrices.count do
           begin
                Matrix(i).Free();
                I := I+ 1;
           end;
     FMaterials.Free;


     FLights.Free;


     FTextures.Free;


     FMatrices.Free;

     FEBRoot.Free();
     Direct3d.Free;
     DirectDraw.Free;

     bInitialized := false;
end;

function TDirect3DIM.AddExecuteBuffer(EBRoot : TExecuteBuffer; iSize : integer) : TExecuteBuffer;
var
   EBTemp : TExecuteBuffer;
begin
     if EBRoot = nil then
        begin
             EBTemp := TExecuteBuffer.Create(Direct3D.D3DDevice, Direct3D.D3DViewport, iSize, nil);
             FEBRoot := EbTemp;
             Result := EBTemp;
        end
     else
         begin
              Result := EBRoot.AddChild(iSize);
          end;
end;

function TDirect3DIM.GetEBRoot : TExecuteBuffer;
begin
     Result := FEbRoot;
end;

procedure TDirect3DIM.RemoveEB(EBDel : TExecuteBuffer);
begin
     EBDel.Free();
end;

function TDirect3DIM.GetEBTagged(EBRoot : TExecuteBuffer; itag : Integer) : TExecuteBuffer;
var
   i : Integer;
   EbTemp : TExecuteBuffer;
begin
     if EbRoot = nil then
        GetEBTagged(FEBRoot,itag);

     if itag = EBRoot.tag then
        begin
             Result := EbRoot;
             exit;
        end;

     i := 0;
     while (i < EBRoot.NumChilds) do
           begin
                EBTemp := GetEBTagged(EBRoot.GetChild(i), itag);
                if EBTemp <> nil then
                   begin
                        result := EBTemp;
                        exit;
                   end;

           end;
end;


function TDirect3DIM.Material(n : integer) : T3dMaterial;
begin
     if n > FMaterials.Count then
        begin
             result := nil;
        end
     else if (n = FMaterials.Count) or (n < 0) then
          begin
               MatTemp := T3dMaterial.Create(Direct3d.D3D, Direct3D.D3DDevice);
               FMaterials.Add(matTemp);
               Result := MatTemp;
          end
     else Result := Fmaterials.Items[n];
end;

function TDirect3DIM.Matrix(n : integer) : T3DMatrix;
var
   MaxTemp : T3dMatrix;
begin
     if n > FMatrices.Count then
        begin
             result := nil;
        end
     else if (n = FMatrices.Count) or (n < 0) then
          begin
               MaxTemp := T3dMatrix.Create(Direct3D.D3DDevice);
               FMatrices.Add(maxTemp);
               Result := MaxTemp;
          end
     else Result := Fmatrices.Items[n];
end;

function TDirect3DIM.Texture(n : integer) : T3DTexture;
var
   TexTemp : T3dtexture;
begin
     if n > FTextures.Count then
        begin
             result := nil;
        end
     else if (n = FTextures.Count) or (n < 0) then
          begin
               TexTemp := T3dTexture.Create(DirectDraw, Direct3D);
               FTextures.Add(TexTemp);
               Result := TexTemp;
          end
     else Result := FTextures.Items[n];
end;

function TDirect3DIM.Light(n : integer) : T3DLight;
var
   LightTemp : T3DLight;
begin
     if n > FLights.Count then
        begin
             result := nil;
        end
     else if (n = FLights.Count) or (n < 0) then
          begin
               LightTemp := T3dLight.Create(Direct3D.D3D);
               FLights.Add(LightTemp);
               Result := LightTemp;
          end
     else Result := FLights.Items[n];
end;

function TDirect3DIM.Execute : Boolean;
begin
  if FEBRoot <> nil then
   begin
     Direct3D.D3DViewPort.Clear(1,Screenrect,D3DCLEAR_TARGET + D3DCLEAR_ZBUFFER);  //
     Result := FEbRoot.Execute;
     DirectDraw.flip;
   end;
end;

procedure TDirect3DIM.WMSize(var Message: TWMSize);
begin
    if (DirectDraw <> nil) and (Not bFullScreen) then
     DirectDraw.WindowMove;
    inherited;
end;


procedure TDirect3DIM.WMMove(var Message: TWMMove);
begin
   if (DirectDraw <> nil) and (not bFullScreen) then
     DirectDraw.WindowMove;
   inherited;
end;

procedure Register;
begin
  RegisterComponents('System', [TDirect3DIM]);
end;

end.
